@@ -240,9 +240,12 @@
         var exactLabelMatches = [];
         var anyLabelMatches = [];
         findPossibleMatches('//*[@aria-label and contains(@aria-label, "' + escapedText + '")]' +
+                '| //a[@title and contains(@title, "' + escapedText + '")]' +
                 '| //img[@alt and contains(@alt, "' + escapedText + '")]', function(match) {
                     // Add to array depending on if it's an exact or partial match.
-                    var attributeData = match.getAttribute('aria-label') || match.getAttribute('alt');
+                    var attributeData = match.getAttribute('aria-label') ||
+                        match.getAttribute('title') ||
+                        match.getAttribute('alt');
                     if (attributeData.trim() === text) {
                         exactLabelMatches.push(match);
                     } else {
